searchNodes=[{"doc":"/ni:mi:/ - Snapshot testing integrated into ExUnit. Snapshot tests assert that some expression matches a reference value. It's like an ExUnit assert , except that the reference value is managed for you by Mneme. Mneme follows in the footsteps of existing snapshot testing libraries like Insta (Rust), expect-test (OCaml), and assert_value (Elixir). Instead of simple value or string comparison, however, Mneme leans heavily into pattern matching. Example Let's say you've written a test for a function that removes even numbers from a list: test &quot;drop_evens/1 should remove all even numbers from an enum&quot; do auto_assert drop_evens ( 1 .. 10 ) auto_assert drop_evens ( [ ] ) auto_assert drop_evens ( [ :a , :b , 2 , :c ] ) end The first time you run this test, you'll see interactive prompts for each call to auto_assert showing a diff and asking if you'd like to accept the generated pattern. After accepting them, your test is updated: test &quot;drop_evens/1 should remove all even numbers from an enum&quot; do auto_assert [ 1 , 3 , 5 , 7 , 9 ] &lt;- drop_evens ( 1 .. 10 ) auto_assert [ ] &lt;- drop_evens ( [ ] ) auto_assert [ :a , :b , :c ] &lt;- drop_evens ( [ :a , :b , 2 , :c ] ) end The next time you run this test, you won't receive a prompt and these will act (almost) like any other assertion. If the result of the call ever changes, you'll be prompted again and can choose to update the test or reject it and let it fail. With a few exceptions, auto_assert/1 acts very similarly to a normal assert . See the macro docs for a list of differences. Quick start Add :mneme do your deps in mix.exs : defp deps do [ { :mneme , &quot;~&gt; 0.0.5&quot; , only : :test } ] end Add :mneme to your :import_deps in .formatter.exs : [ import_deps : [ :mneme ] , inputs : [ &quot;{mix,.formatter}.exs&quot; , &quot;{config,lib,test}/**/*.{ex,exs}&quot; ] ] Start Mneme right after you start ExUnit in test/test_helper.exs : ExUnit . start ( ) Mneme . start ( ) Add use Mneme wherever you use ExUnit.Case : defmodule MyTest do use ExUnit.Case , async : true use Mneme test &quot;arithmetic&quot; do # use auto_assert instead of ExUnit&#39;s assert - run this test # and delight in all the typing you don&#39;t have to do auto_assert 2 + 2 end end Match patterns Mneme tries to generate match patterns that are equivalent to what a human (or at least a nice LLM) would write. Basic data types like strings, numbers, lists, tuples, etc. will be as you would expect. Some values, however, do not have a literal representation that can be used in a pattern match. Pids are such an example. For those, guards are used: auto_assert self ( ) # after running the test and accepting the change auto_assert pid when is_pid ( pid ) &lt;- self ( ) Additionally, local variables can be found and pinned as a part of the pattern. This keeps the number of hard-coded values down, reducing the likelihood that tests have to be updated in the future. test &quot;create_post/1 creates a new post with valid attrs&quot; , %{ user : user } do valid_attrs = %{ title : &quot;my_post&quot; , author : user } auto_assert create_post ( valid_attrs ) end # after running the test test &quot;create_post/1 creates a new post with valid attrs&quot; , %{ user : user } do valid_attrs = %{ title : &quot;my_post&quot; , author : user } auto_assert { :ok , % Post { title : &quot;my_post&quot; , author : ^ user } } &lt;- create_post ( valid_attrs ) end In many cases, multiple valid patterns will be possible. Usually, the &quot;simplest&quot; pattern will be selected by default when you are prompted, but you can cycle through the options as well. Non-exhaustive list of special cases Pinned variables are generated by default if a value is equal to a variable in scope. Date and time values are written using their sigil representation. Struct patterns only include fields that are different from the struct defaults. Structs defined by Ecto schemas exclude primary keys, association foreign keys, and auto generated fields like :inserted_at and :updated_at . This is because these fields are often randomly generated and would fail on subsequent tests. Configuration There are a few controls that can be used to change Mneme's behavior when it runs auto-assertions. These can be set at the module-level by passing options to use Mneme , the describe level using the @mneme_describe attribute, or the test level using the @mneme attribute. For instance: defmodule MyTest do use ExUnit.Case # reject all changes to auto-assertions by default use Mneme , action : :reject test &quot;this test will fail&quot; do auto_assert 1 + 1 end describe &quot;some describe block&quot; do # accept all changes to auto-assertions in this describe block @mneme_describe action : :accept test &quot;this will update without prompting&quot; do auto_assert 2 + 2 end # prompt for any changes in this test @mneme action : :prompt test &quot;this will prompt before updating&quot; do auto_assert 3 + 3 end end end Options :action - The action to be taken when an auto-assertion updates. Actions are one of :prompt , :accept , or :reject . If CI=true is set in environment variables, the action will always be :reject . The default value is :prompt . :target - The target output for auto-assertions. If :mneme , the expression will remain an auto-assertion. If :ex_unit , the expression will be rewritten as an ExUnit assertion. The default value is :mneme . Formatting Mneme uses Rewrite to update source source code, formatting that code before saving the file. Currently, the Elixir formatter and FreedomFormatter are supported. If you do not use a formatter, the first auto-assertion will reformat the entire file. Continuous Integration In a CI environment, Mneme will not attempt to prompt and update any assertions. This behavior is enabled by the CI environment variable, which is set by convention by many continuous integration providers. export CI=true Editor support Guides for optional editor integration can be found here: VS Code","ref":"Mneme.html","title":"Mneme","type":"module"},{"doc":"Sets up Mneme configuration for this module and imports auto_assert/1 . This macro accepts all options described in the &quot;Configuration&quot; section above. Example defmodule MyTest do use ExUnit.Case use Mneme # &lt;- add this test &quot;...&quot; do auto_assert ... end end","ref":"Mneme.html#__using__/1","title":"Mneme.__using__/1","type":"macro"},{"doc":"Generate or run an assertion. auto_assert generates assertions when tests run, issuing a terminal prompt before making any changes (unless configured otherwise). auto_assert [ 1 , 2 ] ++ [ 3 , 4 ] # after running the test and accepting the change auto_assert [ 1 , 2 , 3 , 4 ] &lt;- [ 1 , 2 ] ++ [ 3 , 4 ] If the match no longer succeeds, a warning and new prompt will be issued to update it to the new value. auto_assert [ 1 , 2 , 3 , 4 ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] # after running the test and accepting the change auto_assert [ 1 , 2 , :a , :b ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] Prompts are only issued if the pattern doesn't match the value, so that pattern can also be changed manually. # this assertion succeeds, so no prompt is issued auto_assert [ 1 , 2 , | _ ] &lt;- [ 1 , 2 ] ++ [ :a , :b ] Differences from ExUnit assert The auto_assert macro is meant to match assert as closely as possible. In fact, it generates ExUnit assertions under the hood. There are, however, a few small differences to note: Pattern-matching assertions use the &lt;- operator instead of the = match operator. Value-comparison assertions still use == (for instance, when the expression returns nil or false ). Guards can be added with a when clause, while assert would require a second assertion. For example: auto_assert pid when is_pid ( pid ) &lt;- self ( ) assert pid = self ( ) assert is_pid ( pid ) Bindings in an auto_assert are not available outside of that assertion. For example: auto_assert pid when is_pid ( pid ) &lt;- self ( ) pid # ERROR: pid is not bound If you need to use the result of the assertion, it will evaluate to the expression's value. pid = auto_assert pid when is_pid ( pid ) &lt;- self ( ) pid # pid is the result of self()","ref":"Mneme.html#auto_assert/1","title":"Mneme.auto_assert/1","type":"macro"},{"doc":"Starts Mneme to run auto-assertions as they appear in your tests. This will almost always be added to your test/test_helper.exs , just below the call to ExUnit.start() : # test/test_helper.exs ExUnit . start ( ) Mneme . start ( )","ref":"Mneme.html#start/0","title":"Mneme.start/0","type":"function"},{"doc":"","ref":"Mneme.AssertionError.html","title":"Mneme.AssertionError","type":"exception"},{"doc":"If you are using Visual Studio Code and the ElixirLS extension, there are a couple of things that you can do to smooth out the workflow of running and updating your tests. The ElixirLS extension comes with some built-in tasks to run tests: mix: Run tests mix: Run tests in current file mix: Run test at cursor These are accessible by running the command Run task and then typing typing one of the tasks. By default, these will open a terminal panel, but will not focus it, which isn't optimal as Mneme requires input when something changes. We can bind these tasks to a keyboard shortcut globally and then update the focus defaults on a per-project basis.","ref":"vscode_setup.html","title":"VS Code","type":"extras"},{"doc":"Run the command Preferences: Open Keyboard Shortcuts (JSON) , then copy the following keyboard shortcuts to the bottom of the list: { &quot;key&quot; : &quot;ctrl+space a&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run tests&quot; } , { &quot;key&quot; : &quot;ctrl+space f&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run tests in current file&quot; } , { &quot;key&quot; : &quot;ctrl+space c&quot; , &quot;command&quot; : &quot;workbench.action.tasks.runTask&quot; , &quot;args&quot; : &quot;mix: Run test at cursor&quot; } , You can replace &quot;key&quot; with whatever you'd prefer.","ref":"vscode_setup.html#keyboard-shortcuts","title":"VS Code - Keyboard shortcuts","type":"extras"},{"doc":"Tasks provided by extensions can be modified by adding a .vscode/tasks.json to your project's root. The following will modify the tasks listed above to give input focus to the panel they're run in. { &quot;version&quot; : &quot;2.0.0&quot; , &quot;tasks&quot; : [ { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run tests&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run tests in current file&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , { &quot;group&quot; : &quot;test&quot; , &quot;type&quot; : &quot;mix&quot; , &quot;task&quot; : &quot;Run test at cursor&quot; , &quot;presentation&quot; : { &quot;focus&quot; : true , } , } , ] , } The Mneme repository has this set up here , for example.","ref":"vscode_setup.html#per-project-task-defaults","title":"VS Code - Per-project task defaults","type":"extras"}]